@@ -19,7 +19,6 @@
 #' constraint <- StringToPhyDat('0000?1111 000111111 0000??110', tips, FALSE)
 #' plot(ImposeConstraint(tree, constraint))
 #' @template MRS
-#' @importFrom ape read.tree
 #' @family tree manipulation
 #' @export
 ImposeConstraint <- function (tree, constraint) {
@@ -56,8 +55,8 @@ ImposeConstraint <- function (tree, constraint) {
 
   }
 
-  backbone <- Preorder(RenumberTips(read.tree(
-    text = paste0('(', paste0(rownames(const), collapse = ','), ');')),
+  backbone <- Preorder(RenumberTips(
+    .TextToTree('(', paste0(rownames(const), collapse = ','), ');'),
     tips))
 
   .ChildAtEnd <- function (x) {
@@ -89,6 +88,17 @@ ImposeConstraint <- function (tree, constraint) {
   backbone
 }
 
+#' @importFrom ape read.tree
+.TextToTree <- function (...) {
+  space <- 'XXTREETOOLSSPACEXX'
+  text <- gsub(' ', space, paste0(...), fixed = TRUE)
+  tree <- read.tree(text = text)
+  tree$tip.label <- gsub(space, ' ', tree$tip.label, fixed = TRUE)
+
+  # Return:
+  tree
+}
+
 #' @describeIn ImposeConstraint Expand a constraint to include unconstrained
 #' taxa.
 #' @param toAdd Character vector specifying taxa to add to constraint.
