@@ -2,8 +2,8 @@ preorder_edges_and_nodes <- function(parent, child) {
   .Call(`_TreeTools_preorder_edges_and_nodes`, parent, child)
 }
 
-postorder_edges <- function(edge) {
-  .Call(`_TreeTools_postorder_edges`, edge)
+postorder_edges <- function(edge, sizeSort) {
+  .Call(`_TreeTools_postorder_edges`, edge, sizeSort)
 }
 
 root_binary <- function(edge, outgroup) {
