@@ -93,7 +93,7 @@ RenumberEdges <- function (parent, child, ...) {
 #' functions to the corresponding functions in 'ape'. Single nodes may
 #' need to be collapsed using [ape::collapse.singles] first.  'ape' functions
 #' can cause crashes if nodes are numbered unconventionally -- sometimes
-#' encountered after using tree rearrangement functions, e.g. `phangorn::SPR`.
+#' encountered after using tree rearrangement functions, e.g. `phangorn::SPR()`.
 #'
 #' `Preorder()` is more robust: it supports polytomies, nodes can be numbered
 #' in any sequence, and edges can be listed in any order in the input tree.
@@ -488,6 +488,9 @@ RenumberTips.phylo <- function (tree, tipOrder) {
 #' @export
 RenumberTips.multiPhylo <- function (tree, tipOrder) {
   tree[] <- lapply(tree, RenumberTips.phylo, tipOrder)
+  if (!is.null(attr(tree, 'TipLabel'))) {
+    attr(tree, 'TipLabel') <- TipLabels(tipOrder)
+  }
   tree
 }
 
