@@ -37,13 +37,24 @@
 #' @family tree manipulation
 #'
 #' @template MRS
-#' @importFrom phangorn Ancestors Descendants
 #' @importFrom ape root
 #' @export
-RootTree <- function (tree, outgroupTips) UseMethod('RootTree')
+RootTree <- function (tree, outgroupTips) {
+  if (missing(outgroupTips)) return (tree)
+  if (is.null(outgroupTips) ||
+      length(outgroupTips) == 0) {
+    return(tree)
+  }
+  UseMethod('RootTree')
+}
 
 #' @export
 RootTree.phylo <- function (tree, outgroupTips) {
+  if (missing(outgroupTips) ||
+      is.null(outgroupTips) ||
+      length(outgroupTips) == 0) {
+    return(tree)
+  }
   tipLabels <- tree$tip.label
   if (is.character(outgroupTips)) {
     chosenTips <- match(outgroupTips, tipLabels)
@@ -64,10 +75,9 @@ RootTree.phylo <- function (tree, outgroupTips) {
   } else if (length(outgroupTips) == nTip - 1L) {
     outgroup <- setdiff(seq_len(nTip), outgroupTips)
   } else {
-    ancestry <- unlist(Ancestors(tree, outgroupTips))
-    ancestryTable <- table(ancestry)
-    lineage <- as.integer(names(ancestryTable))
-    lca <- max(lineage[ancestryTable == length(outgroupTips)])
+    ancestryTable <- .AncestorTable(tree, outgroupTips)
+    lineage <- ancestryTable[1, ]
+    lca <- max(lineage[ancestryTable[2, ] == length(outgroupTips)])
     nTip <- length(tipLabels)
     rootNode <- nTip + 1L
     if (lca == rootNode) {
@@ -102,14 +112,42 @@ RootTree.phylo <- function (tree, outgroupTips) {
   res
 }
 
+.AncestorTable <- function (tree, outgroupTips) {
+  edge <- tree$edge
+  parent <- edge[, 1]
+  child <- edge[, 2]
+  nVert <- max(parent)
+  parentOf <- rep_len(NA_integer_, nVert)
+  parentOf[child] <- parent
+  
+  counts <- integer(nVert)
+  i <- outgroupTips
+  while (length(i)) {
+    i <- parentOf[i]
+    i <- i[!is.na(i)]
+    for (j in i) {
+      counts[j] <- counts[j] + 1L
+    }
+  }
+  
+  counted <- counts > 0
+  
+  # Return:
+  rbind(node = which(counted),
+        count = counts[counted])
+}
+
 #' @export
 RootTree.matrix <- function (tree, outgroupTips) {
   tree <- Preorder(tree)
+  if (missing(outgroupTips) ||
+      is.null(outgroupTips) ||
+      length(outgroupTips) == 0L) {
+    return(tree)
+  }
   rootNode <- tree[1]
   nNode <- max(tree[, 1]) - rootNode + 1L
-  if (length(outgroupTips) == 0) {
-    stop("No outgroup tips selected")
-  } else if (length(outgroupTips) == 1L) {
+  if (length(outgroupTips) == 1L) {
     outgroup <- outgroupTips
   } else {
     ancestry <- unlist(.AllAncestors(tree)[outgroupTips])
@@ -152,7 +190,9 @@ RootTree.NULL <- function (tree, outgroupTips) NULL
 #' requested `node` and ordered in [`Preorder`].
 #'
 #' @export
-RootOnNode <- function (tree, node, resolveRoot = FALSE) UseMethod('RootOnNode')
+RootOnNode <- function (tree, node, resolveRoot = FALSE) {
+  UseMethod('RootOnNode', tree)
+}
 
 #' @importFrom fastmatch %fin%
 #' @export
@@ -416,6 +456,9 @@ CollapseEdge <- function (tree, edges) {
 #' node.
 #' @param preorder Logical specifying whether to [Preorder] the tree before
 #' dropping tips.  Necessary if a tree's edges may be unconventionally numbered.
+#' @param check Logical specifying whether to check validity of `tip`. If
+#' `FALSE` and `tip` contains entries that do not correspond to leaves of the
+#' tree, undefined behaviour may occur.
 #'
 #' @return `DropTip()` returns a tree of class `phylo`, with the requested
 #' leaves removed.
@@ -428,39 +471,48 @@ CollapseEdge <- function (tree, edges) {
 #' @family tree manipulation
 #' @template MRS
 #' @export
-DropTip <- function (tree, tip, preorder = TRUE) UseMethod("DropTip")
+DropTip <- function (tree, tip, preorder = TRUE, check = TRUE) UseMethod("DropTip")
 
 #' @rdname DropTip
 #' @export
-DropTip.phylo <- function (tree, tip, preorder = TRUE) {
+DropTip.phylo <- function (tree, tip, preorder = TRUE, check = TRUE) {
   if (preorder) {
     tree <- Preorder(tree)
   }
   labels <- tree$tip.label
   nTip <- length(labels)
-  if (is.null(tip) || any(is.na(tip)) || length(tip) == 0) {
-    drop <- character(0)
+  if (is.null(tip) || !length(tip) || any(is.na(tip))) {
+    drop <- logical(0)
   } else if (is.character(tip)) {
-    drop <- match(tip, tree$tip.label)
-    missing <- is.na(drop)
-    if (any(missing)) {
-      warning(paste(tip[missing], collapse = ', '), " not present in tree")
-      drop <- drop[!missing]
+    drop <- match(tip, labels)
+    if (check) {
+      missing <- is.na(drop)
+      if (any(missing)) {
+        warning(paste(tip[missing], collapse = ', '), " not present in tree")
+        drop <- drop[!missing]
+      }
     }
   } else if (is.numeric(tip)) {
-    nNodes <- nTip + tree$Nnode
-    if (any(tip > nNodes)) {
-      warning("Tree only has ", nNodes, " nodes")
-      tip <- tip[tip <= nNodes]
-    }
-    if (any(tip < 1L)) {
-      warning("`tip` must be > 0")
-      tip <- tip[tip > 0L]
-    }
+    if (check) {
+      nNodes <- nTip + tree$Nnode
+      if (any(tip > nNodes)) {
+        warning("Tree only has ", nNodes, " nodes")
+        tip <- tip[tip <= nNodes]
+      }
+      if (any(tip < 1L)) {
+        warning("`tip` must be > 0")
+        tip <- tip[tip > 0L]
+      }
 
-    if (any(tip > nTip)) {
-      drop <- c(tip[tip <= nTip],
-                unlist(Descendants(tree, tip[tip > nTip])))
+      if (any(tip > nTip)) {
+        edge <- tree$edge
+        parent <- edge[, 1]
+        child <- edge[, 2]
+        drop <- c(tip[tip <= nTip],
+                  .DescendantTips(parent, child, nTip, tip[tip > nTip]))
+      } else {
+        drop <- tip
+      }
     } else {
       drop <- tip
     }
@@ -468,26 +520,52 @@ DropTip.phylo <- function (tree, tip, preorder = TRUE) {
     stop("`tip` must be of type character or numeric")
   }
 
-  if (length(drop) > 0) {
-    if (length(drop) == nTip) {
+  nDrop <- length(drop)
+  if (nDrop) {
+    if (nDrop == nTip) {
       return(structure(list(edge = matrix(0, 0, 2), tip.label = character(0),
-                            NNode = 0), class = 'phylo'))
+                            Nnode = 0), class = 'phylo'))
     }
 
     tree$edge <- drop_tip(tree$edge, drop)
     attr(tree, 'order') <- 'preorder'
     tree$tip.label <- labels[-drop]
-    tree$Nnode <- dim(tree$edge)[1] + 1 - (nTip - length(drop))
+    tree$Nnode <- dim(tree$edge)[1] + 1L - (nTip - nDrop)
   }
 
   # Return:
   tree
 }
 
+# nodes must all be internal
+.DescendantTips <- function (parent, child, nTip, nodes, isDesc = logical(nTip)) {
+  newDescs <- child[parent %in% nodes]
+  recurse <- newDescs > nTip
+  
+  # Return:
+  if (any(recurse)) {
+    isDesc[newDescs[!recurse]] <- TRUE
+    .DescendantTips(parent, child, nTip, newDescs[recurse], isDesc)
+  } else {
+    isDesc[newDescs] <- TRUE
+    which(isDesc)
+  }
+}
+
+#' @describeIn DropTip Direct call to `DropTip.phylo()`, to avoid overhead of
+#' querying object's class.
+#' @export
+DropTipPhylo <- DropTip.phylo
+
 #' @rdname DropTip
 #' @export
-DropTip.multiPhylo <- function (tree, tip, preorder = TRUE) {
-  tree[] <- lapply(tree, DropTip, tip, preorder)
+DropTip.multiPhylo <- function (tree, tip, preorder = TRUE, check = TRUE) {
+  at <- attributes(tree)
+  tree <- lapply(tree, DropTip, tip, preorder)
+  attributes(tree) <- at
+  if (!is.null(at$TipLabel)) {
+    attr(tree, 'TipLabel') <- setdiff(at$TipLabel, tip)
+  }
   tree
 }
 
@@ -495,7 +573,7 @@ DropTip.multiPhylo <- function (tree, tip, preorder = TRUE) {
 #' @return `KeepTip()` returns `tree` with all leaves not in `tip` removed,
 #' in preorder.
 #' @export
-KeepTip <- function (tree, tip, preorder = TRUE) {
+KeepTip <- function (tree, tip, preorder = TRUE, check = TRUE) {
   labels <- if (is.character(tip)) {
     TipLabels(tree)
   } else {
@@ -505,7 +583,7 @@ KeepTip <- function (tree, tip, preorder = TRUE) {
     warning("Tips not in tree: ", paste0(setdiff(tip, labels), collapse = ', '))
   }
   keep <- setdiff(labels, tip)
-  DropTip(tree, keep, preorder)
+  DropTip(tree, keep, preorder, check)
 }
 
 #' Generate binary tree by collapsing polytomies
